import argparse
from pathlib import Path

from . import log, project, runner, settings, templates, tools, utils

_logger = log.get_logger()

app_dir = Path.home().joinpath(".playhdl")
user_settings_file = app_dir.joinpath("settings.json")
project_file = Path("playhdl.json")


def _load_settings(settings_file: Path) -> settings.UserSettings:
    """Load user settings"""
    try:
        return settings.load(settings_file)
    except FileNotFoundError:
        _logger.error(f"Settings file '{settings_file}' was not found. Run 'setup' command to create it first.")
        exit(1)


def _load_project(project_file: Path) -> project.Project:
    """Load project"""
    try:
        return project.load(project_file)
    except FileNotFoundError:
        _logger.error(f"Project file '{project_file}' was not found. Run 'init' command to create it first.")
        exit(1)


def _show_run_options(project_descriptor: project.Project) -> None:
    """Show run options"""
    _logger.info("You can run simulation using one of the options below:")
    for uid in project_descriptor.tools:
        _logger.info(f"  playhdl run {uid}")


def cmd_run(args: argparse.Namespace) -> None:
    """Invoke simulation in the current workspace"""
    _logger.debug(f"Execute 'cmd_run' with {args}")

    # Load user settings
    user_settings = _load_settings(user_settings_file)

    # Load project
    project_descriptor = _load_project(project_file)

    if not args.tool:
        _show_run_options(project_descriptor)
        exit(1)

    # Run simulator
    try:
        runner.run(project_descriptor, user_settings, args.tool, args.waves)
    except (ValueError, RuntimeError, FileNotFoundError) as e:
        _logger.error(str(e))
        exit(1)


def _show_init_options() -> None:
    """Show init options"""
    _logger.info("You can initialize project using one of the options below:")
    for kind in templates.DesignKind:
        _logger.info(f"  playhdl init {kind}")


def cmd_init(args: argparse.Namespace) -> None:
    """Initialize workspace in the current folder"""
    _logger.debug(f"Execute 'cmd_init' with {args}")

    if not args.mode:
        _show_init_options()
        exit(1)

    # Load user settings
    user_settings = _load_settings(user_settings_file)

    # Generate code templates
    source_files = templates.generate(args.mode)
    for src in source_files:
        templates.dump(src, query_force_yes=args.query_force_yes)

    # Init project file
    try:
        project_descriptor = project.create(project_file, args.mode, [f.filename for f in source_files], user_settings)
        _logger.info(f"Save project file to '{project_file}' ...")
        project.dump(project_file, project_descriptor, query_force_yes=args.query_force_yes)
    except ValueError as e:
        _logger.error(str(e))
        exit(1)

    # Show run options
    project_descriptor = _load_project(project_file)
    _show_run_options(project_descriptor)


def cmd_setup(args: argparse.Namespace) -> None:
    """Setup configuration file with avaliable EDA"""
    _logger.debug(f"Execute 'cmd_setup' with {args}")
    settings.setup(app_dir, user_settings_file, query_force_yes=args.query_force_yes)


def cmd_info(args: argparse.Namespace) -> None:
    """Print information about tools and configuration"""
    _logger.debug(f"Execute 'cmd_info' with {args}")
    user_settings = _load_settings(user_settings_file)

    available_tools = []
    for uid, tool_settings in user_settings.tools.items():
        available_tools.append(f"{uid:>15}: {tool_settings.bin_dir}")
    newline = "\n"  # f-string expression part cannot include a backslash
    _logger.info(f"Tools available:\n{newline.join(available_tools)}")
    _logger.info(f"Tools compatibility table:\n{tools.get_compatibility_text_table()}")


def parse_args() -> argparse.Namespace:
    """Parse CLI arguments"""
    parser_descr = f"""playhdl {utils.get_pkg_version()}
avaliable commands:
    setup - setup configuration file with avaliable EDA
    init  - initialize workspace in the current folder
    run   - invoke simulation in the current workspace
    info  - print information about tools and configuration

add -h/--help argument to any command to get more information"""

    class CustomFormatter(
        argparse.RawTextHelpFormatter,
        argparse.ArgumentDefaultsHelpFormatter,
    ):
        pass

    parser = argparse.ArgumentParser(
        description=parser_descr,
        formatter_class=CustomFormatter,
    )
    parser.add_argument("-y", dest="query_force_yes", action="store_true", help="answer 'yes' to all queries")

    subparsers = parser.add_subparsers(help="command to process", dest="command", required=True)

    parser_setup = subparsers.add_parser("setup")
    parser_setup.set_defaults(func=cmd_setup)

    parser_init = subparsers.add_parser("init", formatter_class=CustomFormatter)
    parser_init.add_argument(
        "mode",
        nargs="?",
        type=templates.DesignKind,
        choices=list(templates.DesignKind),
        help="design and testbench mode",
    )
    parser_init.set_defaults(func=cmd_init)

    parser_run = subparsers.add_parser("run")
    parser_run.add_argument("tool", nargs="?", type=tools.ToolUid, help="tool for simulation")
    parser_run.add_argument("--waves", action="store_true", help="open waves after simulation ends")
    parser_run.set_defaults(func=cmd_run)

    parser_setup = subparsers.add_parser("info")
    parser_setup.set_defaults(func=cmd_info)

    return parser.parse_args()


def main() -> None:
    """Entry point to CLI of the application"""
    log.init_logger()
    args = parse_args()
    args.func(args)



from __future__ import annotations

import dataclasses
import enum
import shutil
from abc import ABC, abstractmethod
from dataclasses import dataclass
from pathlib import Path
from typing import Any, Dict, List, Optional, Type

from . import templates, utils


@dataclass
class ToolSettings:
    kind: ToolKind
    bin_dir: Path
    env: Dict[str, str] = dataclasses.field(default_factory=dict)
    extras: Dict[str, Any] = dataclasses.field(default_factory=dict)

    def __post_init__(self) -> None:
        self.bin_dir = Path(self.bin_dir)
        self.kind = ToolKind(self.kind)


class ToolKind(utils.ExtendedEnum):
    MODELSIM = enum.auto()
    XCELIUM = enum.auto()
    VERILATOR = enum.auto()
    ICARUS = enum.auto()
    VCS = enum.auto()
    VIVADO = enum.auto()


@dataclass
class ToolScript:
    build: List[str]
    sim: List[str]
    waves: List[str]


ToolUid = str


def find_tool_dir(tool_kind: ToolKind) -> Optional[Path]:
    """Try to find a directory with executables for the provided tool"""
    return _Tool.get_subclass_by_kind(tool_kind).find_bin_dir()


def generate_script(settings: ToolSettings, design_kind: templates.DesignKind, sources: List[str]) -> ToolScript:
    """Generate script for the provided tool and design"""
    return _Tool.get_subclass_by_kind(settings.kind)(settings).generate_script(design_kind, sources)


def get_compatibility_text_table() -> str:
    """Create text table with tool kinds vs design kind compatibility"""
    col_full_w = 15
    col_text_w = col_full_w - 2

    design_kinds_str = [f"{k.value:^{col_text_w}}" for k in templates.DesignKind]
    header = f"| {'':>{col_text_w}} | {' | '.join(design_kinds_str)}" + " |"

    col_dashes = "-" * col_text_w
    divider = f"| {col_dashes} | {' | '.join([col_dashes] * len(design_kinds_str))}" + " |"

    rows = []
    for tool in ToolKind:
        tool_cls = _Tool.get_subclass_by_kind(tool)
        tool_col = f"{tool.value:>{col_text_w}}"
        compat_cols = []
        for design in templates.DesignKind:
            is_compatible = "X" if design in tool_cls.get_supported_design_kinds() else ""
            compat_cols.append(f"{is_compatible:^{col_text_w}}")
        rows.append(f"| {tool_col} | {' | '.join(compat_cols)}" + " |")

    return "\n".join([header, divider] + rows)


class _Tool(ABC):
    """Generic tool"""

    @abstractmethod
    def __init__(self, settings: ToolSettings) -> None:
        self.settings = settings
        if self.settings.kind != self.get_kind():
            raise RuntimeError(
                f"Provided tool kind '{self.settings.kind}' within settings "
                f"doesn't match with expected value of '{self.get_kind()}'"
            )

    @abstractmethod
    def generate_script(self, design_kind: templates.DesignKind, sources: List[str], **kwargs: Any) -> ToolScript:
        """Generate list of commands to perform tool execution"""
        raise NotImplementedError

    @classmethod
    @abstractmethod
    def get_kind(cls) -> ToolKind:
        """Get kind of the tool"""
        raise NotImplementedError

    @classmethod
    @abstractmethod
    def get_base_exe_name(cls) -> str:
        """Get name of the basic executable"""
        raise NotImplementedError

    @classmethod
    @abstractmethod
    def get_supported_design_kinds(cls) -> List[templates.DesignKind]:
        """Get supported design kinds for the tool"""
        raise NotImplementedError

    @classmethod
    def get_subclass_by_kind(cls, tool_kind: ToolKind) -> Type[_Tool]:
        """Get template class according to tool kind"""
        for cls in _Tool.__subclasses__():  # type: ignore
            if cls.get_kind() == tool_kind:
                return cls
        raise ValueError(f"Can't find tool class for tool_kind={tool_kind}")

    @classmethod
    def find_bin_dir(cls) -> Optional[Path]:
        """Try to find a directory with executables for the provided tool"""
        bin_dir = shutil.which(cls.get_base_exe_name())
        return Path(bin_dir).parent if bin_dir else None

    @classmethod
    def _validate_design_kind(cls, design_kind: templates.DesignKind) -> None:
        """Check that this design kind is supported by the tool"""
        if design_kind not in cls.get_supported_design_kinds():
            raise ValueError(f"{cls.get_kind()} doesn't support provided design kind '{design_kind}'")

    @classmethod
    def _patch_sources(cls, sources: List[str]) -> List[str]:
        """Patch paths to sources"""
        return [f"../{s}" for s in sources]

    @classmethod
    def _stringify_sources(cls, sources: List[str], separator: str = " ") -> str:
        """Convert list of sources to a string"""
        return separator.join(sources)


class _Icarus(_Tool):
    """Icarus Verilog"""

    def __init__(self, settings: ToolSettings) -> None:
        super().__init__(settings)

    def generate_script(self, design_kind: templates.DesignKind, sources: List[str], **kwargs: Any) -> ToolScript:
        self._validate_design_kind(design_kind)

        lang_ver = "-g2001"  # verilog by default
        if design_kind == templates.DesignKind.sv:
            lang_ver = "-g2012"

        sources_opts = self._stringify_sources(self._patch_sources(sources))
        build_cmds = [f"iverilog -Wall {lang_ver} {sources_opts} -o tb.out"]
        sim_cmds = ["vvp tb.out"]
        waves_cmds = ["gtkwave tb.vcd"]

        return ToolScript(build=build_cmds, sim=sim_cmds, waves=waves_cmds)

    @classmethod
    def get_supported_design_kinds(cls) -> List[templates.DesignKind]:
        return [templates.DesignKind.verilog, templates.DesignKind.sv]

    @classmethod
    def get_kind(cls) -> ToolKind:
        return ToolKind.ICARUS

    @classmethod
    def get_base_exe_name(cls) -> str:
        return "iverilog"


class _Modelsim(_Tool):
    """Siemens (Mentor Grapthics) Modelsim"""

    def __init__(self, settings: ToolSettings) -> None:
        super().__init__(settings)

    def generate_script(self, design_kind: templates.DesignKind, sources: List[str], **kwargs: Any) -> ToolScript:
        self._validate_design_kind(design_kind)

        vlog_opts = ""
        if design_kind == templates.DesignKind.sv:
            vlog_opts = "-sv"

        build_cmds = ["vlib worklib", "vmap work worklib"]
        for s in self._patch_sources(sources):
            if design_kind in (templates.DesignKind.verilog, templates.DesignKind.sv):
                build_cmds.append(f"vlog {vlog_opts} {s}")

        sim_cmds = ['vsim -c worklib.tb -do "log -r *;run -all"']
        waves_cmds = ["vsim -view vsim.wlf"]

        return ToolScript(build=build_cmds, sim=sim_cmds, waves=waves_cmds)

    @classmethod
    def get_supported_design_kinds(cls) -> List[templates.DesignKind]:
        return [templates.DesignKind.verilog, templates.DesignKind.sv]

    @classmethod
    def get_kind(cls) -> ToolKind:
        return ToolKind.MODELSIM

    @classmethod
    def get_base_exe_name(cls) -> str:
        return "vsim"


class _Xcelium(_Tool):
    """Cadence Xcelium"""

    def __init__(self, settings: ToolSettings) -> None:
        super().__init__(settings)

    def generate_script(self, design_kind: templates.DesignKind, sources: List[str], **kwargs: Any) -> ToolScript:
        self._validate_design_kind(design_kind)

        vlog_opts = ""
        if design_kind == templates.DesignKind.sv:
            vlog_opts = "-sv"

        build_cmds = []
        for s in self._patch_sources(sources):
            if design_kind in (templates.DesignKind.verilog, templates.DesignKind.sv):
                build_cmds.append(f"xmvlog {vlog_opts} {s}")
        build_cmds.append("xmelab -access +rwc -snapshot tbsim tb")

        sim_cmds = ["xmsim tbsim"]

        waves_cmds = [
            'echo "database open -overwrite tb.vcd" > waves.cmd',
            "simvision -input waves.cmd -waves",
        ]

        return ToolScript(build=build_cmds, sim=sim_cmds, waves=waves_cmds)

    @classmethod
    def get_supported_design_kinds(cls) -> List[templates.DesignKind]:
        return [templates.DesignKind.verilog, templates.DesignKind.sv]

    @classmethod
    def get_kind(cls) -> ToolKind:
        return ToolKind.XCELIUM

    @classmethod
    def get_base_exe_name(cls) -> str:
        return "xmsim"


class _Verilator(_Tool):
    """Veripool Verilator"""

    def __init__(self, settings: ToolSettings) -> None:
        super().__init__(settings)

    def generate_script(self, design_kind: templates.DesignKind, sources: List[str], **kwargs: Any) -> ToolScript:
        self._validate_design_kind(design_kind)

        lang_ver = "+verilog2001ext+v"  # verilog by default
        if design_kind == templates.DesignKind.sv:
            lang_ver = "+systemverilogext+sv"

        sources_opts = self._stringify_sources(self._patch_sources(sources))
        build_cmds = [f"verilator {lang_ver} --trace --binary -j 0 {sources_opts}"]
        sim_cmds = ["./obj_dir/Vtb"]
        waves_cmds = ["gtkwave tb.vcd"]

        return ToolScript(build=build_cmds, sim=sim_cmds, waves=waves_cmds)

    @classmethod
    def get_supported_design_kinds(cls) -> List[templates.DesignKind]:
        return [templates.DesignKind.verilog, templates.DesignKind.sv]

    @classmethod
    def get_kind(cls) -> ToolKind:
        return ToolKind.VERILATOR

    @classmethod
    def get_base_exe_name(cls) -> str:
        return "verilator"


class _Vcs(_Tool):
    """Synopsys VCS"""

    class _GuiKind(utils.ExtendedEnum):
        DVE = enum.auto()
        VERDI = enum.auto()

    def __init__(self, settings: ToolSettings) -> None:
        super().__init__(settings)
        self.gui = self._GuiKind(settings.extras.get("gui", "verdi"))

    def generate_script(self, design_kind: templates.DesignKind, sources: List[str], **kwargs: Any) -> ToolScript:
        self._validate_design_kind(design_kind)

        vlog_opts = ""
        if design_kind == templates.DesignKind.sv:
            vlog_opts = "-sverilog"
        elif design_kind == templates.DesignKind.sv_uvm12:
            vlog_opts = "-sverilog -ntb_opts uvm-1.2"

        sources_opts = self._stringify_sources(self._patch_sources(sources))
        build_cmds = [f"vcs -full64 {vlog_opts} -debug_acc+all +vcs+vcdpluson +vcs+fsdbon {sources_opts}"]

        sim_cmds = ["./simv"]

        if self.gui == self._GuiKind.VERDI:
            waves_cmds = ["verdi -ssf novas.fsdb"]
        else:
            waves_cmds = ["dve -vpd vcdplus.vpd"]

        return ToolScript(build=build_cmds, sim=sim_cmds, waves=waves_cmds)

    @classmethod
    def get_supported_design_kinds(cls) -> List[templates.DesignKind]:
        return [
            templates.DesignKind.verilog,
            templates.DesignKind.sv,
            templates.DesignKind.sv_uvm12,
        ]

    @classmethod
    def get_kind(cls) -> ToolKind:
        return ToolKind.VCS

    @classmethod
    def get_base_exe_name(cls) -> str:
        return "vcs"


class _Vivado(_Tool):
    """Xilinx Vivado"""

    def __init__(self, settings: ToolSettings) -> None:
        super().__init__(settings)

    def generate_script(self, design_kind: templates.DesignKind, sources: List[str], **kwargs: Any) -> ToolScript:
        self._validate_design_kind(design_kind)

        uvm_vlog_opts = ""
        uvm_elab_opts = ""
        if design_kind == templates.DesignKind.sv_uvm12:
            uvm_vlog_opts = "-uvm_version 1.2 -L uvm"
            uvm_elab_opts = "-L uvm"

        build_cmds = []
        for s in self._patch_sources(sources):
            if design_kind == templates.DesignKind.verilog:
                build_cmds.append(f"xvlog -work worklib {s}")
            elif design_kind in [templates.DesignKind.sv, templates.DesignKind.sv_uvm12]:
                build_cmds.append(f"xvlog -work worklib -sv {uvm_vlog_opts} {s}")
        build_cmds.append(f"xelab worklib.tb {uvm_elab_opts} --debug all -s tbsim")

        sim_cmds = [
            'echo "log_wave -recursive *;run all;quit" > sim.tcl',
            "xsim tbsim --wdb tb.wdb --t sim.tcl",
        ]

        waves_cmds = [
            'echo "open_wave_database tb.wdb" > waves.tcl',
            "vivado -source waves.tcl",
        ]

        return ToolScript(build=build_cmds, sim=sim_cmds, waves=waves_cmds)

    @classmethod
    def get_supported_design_kinds(cls) -> List[templates.DesignKind]:
        return [
            templates.DesignKind.verilog,
            templates.DesignKind.sv,
            templates.DesignKind.sv_uvm12,
        ]

    @classmethod
    def get_kind(cls) -> ToolKind:
        return ToolKind.VIVADO

    @classmethod
    def get_base_exe_name(cls) -> str:
        return "xsim"



import distutils
import json
from enum import Enum
from pathlib import Path
from typing import Any, Dict, List, Union

import pkg_resources

from . import log

_logger = log.get_logger()


def get_pkg_version() -> str:
    """Get version of the package"""
    return pkg_resources.get_distribution("playhdl").version


class ExtendedJsonEncoder(json.JSONEncoder):
    """Custom encoder that can serialize more types"""

    def default(self, obj: Any) -> Union[str, Any]:
        if isinstance(obj, Path):
            return str(obj)
        elif isinstance(obj, Enum):
            return obj.name
        else:
            return json.JSONEncoder.default(self, obj)


def dump_json(file: Path, data: Dict) -> None:
    """Dump data to a JSON file"""
    with file.open("w") as f:
        json.dump(data, f, cls=ExtendedJsonEncoder, indent=4)


def load_json(file: Path) -> Dict:
    """Load data from a JSON file"""
    with file.open("r") as f:
        return json.load(f)


def is_write_allowed(filepath: Path, force_yes: bool = False) -> bool:
    """Query user if file exists and write it then"""
    if filepath.is_file():
        # Provide message and lamda with dump method in case caller want to query user for further steps
        _logger.warning(f"File '{filepath}' already exists. It will be overwriten!")
        return force_yes or input_query_yes_no()
    else:
        return True


def input_query_yes_no(question: str = "Do you want to proceed?") -> bool:
    """Ask a yes/no question"""
    _logger.warning(f"{question} [y/n]")
    while True:
        try:
            return bool(distutils.util.strtobool(input().lower()))  # type: ignore
        except ValueError:
            _logger.error("Usupported answer! Please type y/yes or n/no.")


class ExtendedEnum(str, Enum):
    def _generate_next_value_(name, start: int, count: int, last_values: List[Any]) -> Any:  # type: ignore
        """This uses enumeration name as a string value for auto() call"""
        return name.lower()

    def __str__(self) -> str:
        return self.value

    @classmethod
    def aslist(cls) -> List[str]:
        """List of values of the enum"""
        return list(map(lambda c: c.value, cls))  # type: ignore



from __future__ import annotations

import enum
from abc import ABC, abstractmethod
from dataclasses import dataclass
from pathlib import Path
from typing import Any, List, Type

from .. import log, utils

_logger = log.get_logger()


class DesignKind(utils.ExtendedEnum):
    verilog = enum.auto()
    sv = enum.auto()
    sv_uvm12 = enum.auto()
    vhdl = enum.auto()


@dataclass
class TemplateDescriptor:
    filename: str
    content: str


def generate(design_kind: DesignKind) -> List[TemplateDescriptor]:
    """Get template files according to design kind"""
    _logger.info(f"Generate templates for '{design_kind}' design ...")
    templates = _DesignTemplate.get_subclass_by_kind(design_kind)().generate()
    filenames = [t.filename for t in templates]
    _logger.info(f"  {' '.join(filenames)}")
    return templates


def dump(template: TemplateDescriptor, **kwargs: Any) -> None:
    """Save template to a disc"""
    filepath = Path(template.filename)
    _logger.info(f"Save '{filepath}' to a disk ...")
    if utils.is_write_allowed(filepath, bool(kwargs.get("query_force_yes", False))):
        with filepath.open("w") as f:
            f.write(template.content)


class _DesignTemplate(ABC):
    """Generic template"""

    def generate(self, **kwargs: Any) -> List[TemplateDescriptor]:
        """Generate design template files"""
        content = self._read_template_file(self.get_template_name())
        return [TemplateDescriptor(self.get_template_name(), content)]

    @classmethod
    @abstractmethod
    def get_kind(cls) -> DesignKind:
        """Get kind of the template"""
        raise NotImplementedError

    @classmethod
    @abstractmethod
    def get_template_name(cls) -> str:
        """Get name of the template"""
        raise NotImplementedError

    def _read_template_file(self, filename: str) -> str:
        """Read provided template file"""
        with Path(__file__).parent.joinpath(filename).open("r") as f:
            return f.read()

    @classmethod
    def get_subclass_by_kind(cls, design_kind: DesignKind) -> Type[_DesignTemplate]:
        """Get template class according to design kind"""
        for cls in _DesignTemplate.__subclasses__():  # type: ignore
            if cls.get_kind() == design_kind:
                return cls
        raise ValueError(f"Can't find template class for design_kind={design_kind}")


class _Verilog(_DesignTemplate):
    """Verilog design template"""

    @classmethod
    def get_kind(cls) -> DesignKind:
        return DesignKind.verilog

    @classmethod
    def get_template_name(cls) -> str:
        return "tb.v"


class _SystemVerilog(_DesignTemplate):
    """SystemVerilog design template"""

    @classmethod
    def get_kind(cls) -> DesignKind:
        """Get kind of the template"""
        return DesignKind.sv

    @classmethod
    def get_template_name(cls) -> str:
        return "tb.sv"


class _SystemVerilogUvm12(_DesignTemplate):
    """SystemVerilog UVM 1.2 design template"""

    @classmethod
    def get_kind(cls) -> DesignKind:
        return DesignKind.sv_uvm12

    @classmethod
    def get_template_name(cls) -> str:
        return "tb_uvm12.sv"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['playhdl', 'playhdl.templates']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['playhdl = playhdl.cli:main']}

setup_kwargs = {
    'name': 'playhdl',
    'version': '0.2.0',
    'description': 'Tool to play with HDL (inspired by EdaPlayground)',
    'long_description': '# playhdl\n\nYou can think about `playhdl` as [EdaPlayground](https://edaplayground.com/), but which is CLI-based and uses simulators on your local machine.\n\nIt gives you ability to simulate tiny snippets of HDL code in several commands without any headache related to vast tool guides, command-line arguments and etc.:\n\n```sh\nplayhdl init sv\nplayhdl run modelsim\n```\n\n## Features\n\n`playhdl` is written in pure Python without any external dependencies, so it is easy to use it in any environment (laptop, server, etc.), where Python 3.8+ is available.\n\nIt supports various project types (HDL languages + libraries) and many simulators:\n\n|               |    verilog         |      sv            |   sv_uvm12         |     vhdl      |\n| ------------- | :----------------: | :----------------: | :----------------: | :-----------: |\n| [modelsim](https://eda.sw.siemens.com/en-US/ic/modelsim/) | :heavy_check_mark: | :heavy_check_mark: | | |\n| [xcelium](https://www.cadence.com/ko_KR/home/tools/system-design-and-verification/simulation-and-testbench-verification/xcelium-simulator.html) | :heavy_check_mark: | :heavy_check_mark: | | |\n| [verilator](https://www.veripool.org/verilator/) | :heavy_check_mark: | :heavy_check_mark: | | |\n| [icarus](http://iverilog.icarus.com/) | :heavy_check_mark: | :heavy_check_mark: | | |\n| [vcs](https://www.synopsys.com/verification/simulation/vcs.html) | :heavy_check_mark: | :heavy_check_mark: | :heavy_check_mark: | |\n| [vivado](https://www.xilinx.com/products/design-tools/vivado.html) | :heavy_check_mark: | :heavy_check_mark: | :heavy_check_mark: | |\n\n## Quick start\n\n* Install the latest stable release (Python 3.8+ is required):\n\n```sh\npython -m pip install -U playhdl\n```\n\n* Setup settings file `$HOME/.playhdl/settings.json` with a list of all automatically-discoverd simulators. Edit file manually to add undiscovered ones. This have to be done only once.\n\n```sh\nplayhdl setup\n```\n\n* Initialize project file `playhdl.json` and template testbench in the current directory. Project file contains specific commands to be executed for compilation and simulation processes. Edit it manually to tweak tool arguments if required.\n\n```sh\nplayhdl sv # this will create ready-to-simulate tb.sv\n```\n\n* Run simulation in one of the supported simulators for this project (language):\n\n```sh\nplayhdl run icarus\n# to open waves after simulation\nplayhdl run icarus --waves\n```\n\n## Tool command guide\n\nTo get general help and command list:\n\n```sh\nplayhdl -h\n```\n\nTo get help about specific command\n\n```sh\nplayhdl <command> -h\n```\n\n### `setup` command\n\nSettings of the tool are stored in the JSON file under `$HOME/.playhdl` directory.\n\nTo create `$HOME/.playhdl/settings.json` run\n\n```sh\nplayhdl setup\n```\n\nIt will try to find all supported simulators and fill the json. If you have multiple versions of simulators or some of them were not found, add them manually to your settings file.\n\nSettings file structure:\n\n```json\n{\n    "tools": {\n        "<tool_uid>": {\n            "kind": "<tool_kind>",\n            "bin_dir": "<path_to_bin>",\n            "env": {},\n            "extras": {}\n        }\n    }\n}\n```\n\nAll tools settings are located in a dictionary under `"tools"` key.\n\nEvery tool has it is own `tool_uid`, which is just a string with any unique name, e.g. `"modelsim"`, `"verilator5"`, `"my_secret_simulator"`.\n\nValid `"kind"` must be provided:\n\n* `"modelsim"`\n* `"xcelium"`\n* `"verilator"`\n* `"icarus"`\n* `"vcs"`\n* `"vivado"`\n\nOther fields:\n\n* `"bin_dir"` - a string with a path to a directory with executable files\n* `"env"` - a dictionary with additional enviroment variables (keys and values are strings)\n* `"extras"` - a dictionary with extra values for a specific simulator kind\n\nExtras for `"vcs"` kind:\n\n* `"gui"` - `"verdi"` or `"dve"` select default GUI for VCS\n\n### `init` command\n\nThis command creates JSON project file `playhdl.json` and HDL testbench in the current directory.\n\n```sh\nplayhdl init <mode>\n```\n\nWhere `<mode>` is one of the supported project modes:\n\n* `verilog` - Verilog-2001\n* `sv` - SystemVerilog-2017\n* `sv_uvm12` - SystemVerilog-2017 + UVM 1.2\n* `vhdl` - VHDL-93\n\nProject file is filled with scripts for all suitable simulators for the selected mode. It\'s internal structure:\n\n```json\n{\n    "tools": {\n        "<tool_uid>": {\n            "build": [\n                "<cmd0>",\n                "<cmd1>",\n                "<cmd2>"\n            ],\n            "sim": [\n                "<cmd>"\n            ],\n            "waves": [\n                "<cmd>"\n            ]\n        }\n    }\n}\n```\n\nThere are three categories of commands:\n\n* `"build"` - commands needed to compile and elaborate sources\n* `"sim"` - commands needed to run simulation\n* `"waves"` - commands needed to open waves for analysis\n\nAny command can be customized for specific needs.\n\n### `run` command\n\nThis command runs CLI-mode simulation in a specific simulator according to project file\n\n```sh\nplayhdl run <tool_uid>\n```\n\nArgument `--waves` can be added to open waves for analysis after simulation ends\n\n```sh\nplayhdl run <tool_uid> --waves\n```\n\n### `info` command\n\nThis command just prints some useful information:\n\n* all tools specified in your settings file\n* current compatibility table between project mode and simulator\n\n```sh\nplayhdl info\n```\n\n## Developer guide\n\nInstall `poetry`\n\n```sh\npython -m pip install -U poetry\n```\n\nSetup virtual environment\n\n```sh\nmake setup-dev\n```\n\nYou can specify Python version to use\n\n```sh\nmake setup-dev PYTHON_VERSION=3.9\n```\n\nTo run `playhdl` from sources\n\n```sh\npoetry run playhdl <args>\n```\n\n`Makefile` provides additional ways to interact with project:\n\n* `make format` - auto-format all sources\n* `make check-format` - check current formatting of sources\n* `make lint` - perform linting\n* `make type` - perform type checking\n* `make test` - run all tests\n* `make pre-commit` - shorthand for combination of `check-format`, `lint`, `type`, `test`\n\n## Miscellaneous\n\n### Offline install\n\nFor an offline install you have several options of how to get `wheel`:\n\n* build locally using [poetry](https://python-poetry.org/)\n\n```sh\npython -m pip install -U poetry\npoetry build\n```\n\n* download `.whl` from [PyPi](https://pypi.org/)\n\n```sh\npython -m pip download playhdl\n```\n\nThen you can use pip to install it on an offline machine:\n\n```sh\npython -m pip install <wheel_file_name>.whl\n```\n',
    'author': 'esynr3z',
    'author_email': 'esynr3z@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/esynr3z/playhdl',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
